"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable no-param-reassign */
const readline = require("readline");
const connect_1 = require("../connect");
const index_1 = require("../index");
const errorhandler_1 = require("./errorhandler");
const profiles_1 = require("./profiles");
class XdccJSbin extends profiles_1.default {
    constructor() {
        super();
        const lazyArgs = XdccJSbin.isCopyPasteARGV();
        if (lazyArgs) {
            this.lazy(lazyArgs);
        }
        else {
            const profileAction = this.profileAction();
            if (!profileAction)
                this.main();
        }
    }
    static isLazySyntaxCorrect(match) {
        if (match[1].toLowerCase() === 'msg' && match[3].toLowerCase() === 'xdcc' && match[4].toLowerCase() === 'send') {
            return true;
        }
        return false;
    }
    lazy(match) {
        if (!this.defaultProfile) {
            throw new errorhandler_1.default('%info% You need to setup a profile first in order to use copy paste method');
        }
        else if (XdccJSbin.isLazySyntaxCorrect(match)) {
            const [, , bot, , , download] = match;
            this.program.bot = bot;
            this.program.download = [download];
            this.downloadWith(this.defaultProfile);
        }
        else {
            throw new errorhandler_1.default('%danger% Wrong format, try (with double quotes): %grey%"/MSG My|BOT xdcc send 23, 25, 50-55"');
        }
    }
    static isCopyPasteARGV() {
        const match = /\/(msg|MSG) (.*) (xdcc|XDCC) (send|SEND) (.*)$/
            .exec(process.argv.join(' '));
        if (match) {
            return match;
        }
        return undefined;
    }
    main() {
        if (this.defaultProfile)
            return this.downloadWith(this.defaultProfile);
        return this.downloadWith(this.xdccBINOPTS());
    }
    writeMSG(time) {
        if (!this.program.quiet) {
            readline.cursorTo(process.stderr, 1);
            process.stderr.write(connect_1.default.replace(`%info% waiting: ${time}`));
            readline.clearLine(process.stderr, 1);
        }
    }
    clearMSG() {
        if (!this.program.quiet) {
            readline.clearLine(process.stderr, 0);
            readline.cursorTo(process.stderr, 0);
        }
    }
    waitMessage(time, xdccJS, bot, download) {
        const start = time;
        const inter = setInterval(async () => {
            time -= 1;
            if (start > 0) {
                this.writeMSG(time);
            }
            if (time <= 0) {
                if (start > 0) {
                    this.clearMSG();
                }
                const job = await xdccJS.download(bot, download, { ipv6: this.program.ipv6, throttle: this.program.throttle });
                if (!this.program.path) {
                    job.on('pipe', (stream) => {
                        stream.pipe(process.stdout);
                    });
                }
                clearInterval(inter);
            }
        }, 1000);
    }
    downloadWith(opts) {
        if (!this.hasProfileAction()) {
            if (!this.program.bot && !opts[1].bot) {
                throw new errorhandler_1.default('%danger% Missing bot name, eg. %grey%--bot "XDCC|BOT"');
            }
            if (!this.program.download) {
                throw new errorhandler_1.default('%danger% You must specify a packet number to download, eg. %grey%--download 1, 3, 55-60');
            }
            const download = this.program.download.join('');
            const bot = this.program.bot ? this.program.bot : opts[1].bot;
            const wait = opts[1].wait || 0;
            if (!bot)
                throw new Error('Control flow error: downloadwith()');
            const xdccJS = new index_1.default(opts[0])
                .on('ready', () => {
                this.waitMessage(wait, xdccJS, bot, download);
            })
                .on('can-quit', () => {
                xdccJS.quit();
            })
                .on('error', (e) => {
                if (!e) {
                    console.error(new errorhandler_1.default('%danger% Unknown error'));
                    return;
                }
                // trick to avoid "hard" throws
                const error = new errorhandler_1.default(`%danger% ${e.message}`);
                console.error(error.message);
            });
        }
    }
}
exports.default = XdccJSbin;
