"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CtcpParser = void 0;
/* eslint-disable max-len */
/* eslint-disable no-bitwise */
/* eslint-disable no-param-reassign */
const fs = require("fs");
const path = require("path");
const addjob_1 = require("./addjob");
class CtcpParser extends addjob_1.default {
    path;
    botNameMatch;
    resumequeue = [];
    constructor(params) {
        super(params);
        this.botNameMatch = params.botNameMatch;
        this.path = params.path;
        this.on('ctcp request', (resp) => {
            const isDownloadRequest = this.checkBeforeDL(resp, this.candidates[0]);
            if (isDownloadRequest) {
                this.emit('debug', 'xdccJS:: BEFORE_TCP_OK');
                this.emit('prepareDL', isDownloadRequest);
            }
        });
    }
    static pathCheck(fpath) {
        if (typeof fpath === 'string') {
            const tmp = path.normalize(fpath);
            if (path.isAbsolute(tmp)) {
                CtcpParser.mkdir(tmp);
                return tmp;
            }
            CtcpParser.mkdir(path.join(path.resolve('./'), fpath));
            return path.join(path.resolve('./'), fpath);
        }
        return false;
    }
    static mkdir(fpath) {
        if (!fs.existsSync(fpath)) {
            fs.mkdirSync(fpath, {
                recursive: true,
            });
        }
    }
    SecurityCheck(nick, candidate) {
        if (candidate) {
            nick = nick.toLowerCase();
            candidate.nick = candidate.nick.toLowerCase();
            candidate.cancelNick = nick;
            if (this.botNameMatch) {
                if (nick === candidate.nick) {
                    return true;
                }
                return false;
            }
            return true;
        }
        return false;
    }
    checkBeforeDL(resp, candidate) {
        const fileInfo = this.parseCtcp(resp.message, resp.nick);
        if (fileInfo && this.SecurityCheck(resp.nick, candidate)) {
            this.SetupTimeout({
                candidate,
                eventType: 'error',
                message: `couldn't connect to %yellow%${fileInfo.ip}:${fileInfo.port}`,
                padding: 6,
                delay: this.timeout,
                fileInfo,
            });
            if (fileInfo.type === 'DCC SEND') {
                const isResume = this.checkExistingFiles(fileInfo, candidate, resp);
                if (!isResume)
                    return { fileInfo, candidate };
            }
        }
        return undefined;
    }
    addToResumeQueue(fileInfo, nick) {
        this.resumequeue.push({
            type: fileInfo.type,
            nick,
            ip: fileInfo.ip,
            length: fileInfo.length,
            token: fileInfo.token,
            position: fileInfo.position,
            port: fileInfo.port,
            filePath: fileInfo.filePath,
            file: fileInfo.file,
        });
    }
    checkExistingFiles(fileInfo, candidate, resp) {
        if (fs.existsSync(fileInfo.filePath) && this.path) {
            fileInfo.position = fs.statSync(fileInfo.filePath).size - 8192;
            if (fileInfo.position < 0) {
                fileInfo.position = 0;
            }
            fileInfo.length -= fileInfo.position;
            const quotedFilename = CtcpParser.fileNameWithQuotes(fileInfo.file);
            this.ctcpRequest(resp.nick, 'DCC RESUME', quotedFilename, fileInfo.port, fileInfo.position, fileInfo.token);
            this.addToResumeQueue(fileInfo, resp.nick);
            this.SetupTimeout({
                candidate,
                eventType: 'error',
                message: `couldn't resume download of %cyan%${fileInfo.file}`,
                padding: 6,
                delay: this.timeout,
                fileInfo,
            });
            this.emit('debug', 'xdccJS:: BEFORE_TCP_REQUEST_RESUME');
            return true;
        }
        return false;
    }
    static fileNameWithQuotes(string) {
        if (/\s/.test(string)) {
            return `"${string}"`;
        }
        return string;
    }
    static ctcpMatch(text) {
        const match = text.match(/(?:[^\s"]+|"[^"]*")+/g);
        if (match === null) {
            throw new TypeError(`CTCP : received unexpected msg : ${text}`);
        }
        else {
            return match;
        }
    }
    fileInfoBuilder(parts, resume) {
        if (resume) {
            return {
                type: `${parts[0]} ${parts[1]}`,
                file: parts[2].replace(/"/g, ''),
                filePath: resume.filePath,
                ip: resume.ip,
                port: resume.port,
                position: resume.position,
                length: resume.length,
                token: resume.token,
            };
        }
        return {
            type: `${parts[0]} ${parts[1]}`,
            file: parts[2].replace(/"/g, ''),
            filePath: this.path ? path.normalize(`${this.path}/${parts[2].replace(/"/g, '')}`) : 'pipe',
            ip: CtcpParser.uint32ToIP(parts[3]),
            port: parseInt(parts[4], 10),
            position: 0,
            length: parseInt(parts[5], 10),
            token: parts[6],
        };
    }
    parseCtcp(text, nick) {
        const parts = CtcpParser.ctcpMatch(text);
        const type = `${parts[0]} ${parts[1]}`;
        if (type === 'DCC ACCEPT') {
            this.emit('debug', 'xdccJS:: BEFORE_TCP_RESUME_ACCEPT');
            const resume = this.resumequeue.filter((q) => q.nick === nick);
            this.resumequeue = this.resumequeue.filter((q) => q.nick !== nick);
            if (resume.length) {
                return this.fileInfoBuilder(parts, resume[0]);
            }
        }
        if (type === 'DCC SEND') {
            this.emit('debug', 'xdccJS:: BEFORE_TCP_ACCEPT');
            return this.fileInfoBuilder(parts);
        }
        return undefined;
    }
    static uint32ToIP(value) {
        const n = parseInt(value, 10);
        if (Number.isNaN(n)) {
            return value;
        }
        const byte1 = n & 255;
        const byte2 = (n >> 8) & 255;
        const byte3 = (n >> 16) & 255;
        const byte4 = (n >> 24) & 255;
        return `${byte4}.${byte3}.${byte2}.${byte1}`;
    }
}
exports.CtcpParser = CtcpParser;
