"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable max-len */
/* eslint-disable no-param-reassign */
const fs = require("fs");
const net = require("net");
const stream_1 = require("stream");
const stream_throttle_1 = require("stream-throttle");
const ctcp_parser_1 = require("./ctcp_parser");
const get_ip_1 = require("./lib/get-ip");
const ProgressBar = require("./lib/progress");
class Downloader extends ctcp_parser_1.CtcpParser {
    passivePort;
    ip;
    throttle;
    constructor(params) {
        super(params);
        this.ip = Downloader.getIp();
        this.passivePort = params.passivePort;
        this.throttle = params.throttle;
        this.on('prepareDL', (downloadrequest) => {
            this.prepareDL(downloadrequest);
        });
    }
    static async getIp() {
        const ip = await (0, get_ip_1.default)();
        let v4;
        if (ip.v4) {
            const d = ip.v4.split('.');
            const results = ((+d[0] * 256 + +d[1]) * 256 + +d[2]) * 256 + +d[3];
            v4 = `${results}`;
        }
        return { v4, v6: ip.v6 };
    }
    setupStream(fileInfo) {
        if (this.path) {
            if (fileInfo.type === 'DCC ACCEPT') {
                return fs.createWriteStream(fileInfo.filePath, {
                    flags: 'r+',
                    start: fileInfo.position,
                });
            }
            if (fileInfo.type === 'DCC SEND') {
                return fs.createWriteStream(fileInfo.filePath);
            }
            throw Error('Error in control flow: setupStream');
        }
        else {
            return new stream_1.PassThrough();
        }
    }
    prepareDL(downloadrequest) {
        const { fileInfo } = downloadrequest;
        const { candidate } = downloadrequest;
        const stream = this.setupStream(fileInfo);
        if (fileInfo.port === 0) {
            this.emit('debug', 'xdccJS:: TCP_INCOMING_READY');
            const pick = this.portPicker();
            const server = net.createServer((client) => {
                this.SetupTimeout({
                    candidate,
                    eventType: 'error',
                    message: '%danger% Timeout: no initial connnection',
                    delay: this.timeout,
                    disconnectAfter: {
                        stream,
                        server,
                        socket: client,
                        pick,
                    },
                    padding: 6,
                    fileInfo,
                });
                this.processDL(server, client, stream, candidate, fileInfo, pick);
            });
            const listenIp = candidate.opts && candidate.opts.ipv6 ? '::' : '0.0.0.0';
            server.listen(pick, listenIp, () => {
                this.ip.then((ip) => {
                    this.raw(`PRIVMSG ${candidate.nick} ${String.fromCharCode(1)}DCC SEND ${fileInfo.file} ${ip} ${pick} ${fileInfo.length} ${fileInfo.token}${String.fromCharCode(1)}`);
                }).catch((e) => {
                    const pass = {
                        server,
                        stream,
                        candidate,
                        fileInfo,
                        pick,
                        received: 0,
                        bufferType: fileInfo.length > 4294967295 ? '64bit' : '32bit',
                    };
                    this.onError(pass, e);
                });
            });
        }
        else {
            this.emit('debug', 'xdccJS:: TCP_OUTGOING_READY');
            const client = net.connect(fileInfo.port, fileInfo.ip);
            this.processDL(undefined, client, stream, candidate, fileInfo, undefined);
        }
    }
    portPicker() {
        const available = this.passivePort.filter((ports) => !this.portInUse.includes(ports));
        const pick = available[Math.floor(Math.random() * available.length)];
        this.portInUse.push(pick);
        return pick;
    }
    processDL(server, client, stream, candidate, fileInfo, pick) {
        candidate.cancel = this.makeCancelable(candidate, client);
        this.print(`%info% downloading : %cyan%${fileInfo.file}`, 5);
        const bar = Downloader.setupProgressBar(fileInfo.length);
        const pass = {
            server,
            client,
            stream,
            candidate,
            fileInfo,
            pick,
            bar,
            received: 0,
            bufferType: fileInfo.length > 4294967295 ? '64bit' : '32bit',
        };
        client.setTimeout(this.timeout);
        client.on('timeout', () => this.onTimeOut(pass));
        client.on('error', (e) => this.onError(pass, e));
        const sendBuffer = Buffer.alloc(pass.bufferType === '64bit' ? 8 : 4);
        const { throttle } = { ...candidate.opts } || { ...this };
        if (throttle) {
            const tg = new stream_throttle_1.ThrottleGroup({ rate: throttle });
            const thr = client.pipe(tg.throttle({ rate: throttle }));
            thr.on('data', (data) => this.onData(pass, data, sendBuffer));
        }
        else {
            client.on('data', (data) => this.onData(pass, data, sendBuffer));
        }
        client.once('data', () => this.emit('debug', 'xdccJS:: TCP_DOWNLOADING'));
        client.on('close', (e) => this.onClose(pass, e));
    }
    onTimeOut(args) {
        if (args.received === args.fileInfo.length)
            return;
        this.emit('debug', 'xdccJS:: TCP_TIMEOUT');
        this.SetupTimeout({
            candidate: args.candidate,
            eventType: 'error',
            message: 'Timeout',
            delay: 0,
            disconnectAfter: {
                stream: args.stream,
                server: args.server,
                socket: args.client,
                pick: args.pick,
                bar: args.bar,
            },
            padding: 6,
            fileInfo: args.fileInfo,
            executeLater: () => {
                this.redownload(args.candidate, args.fileInfo);
            },
        });
    }
    onError(args, e) {
        if (args.received === args.fileInfo.length)
            return;
        if (e.message === 'cancel')
            this.emit('debug', 'xdccJS:: TCP_CANCEL');
        else
            this.emit('debug', `xdccJS:: TCP_ERROR @ ${e.message}`);
        this.SetupTimeout({
            candidate: args.candidate,
            eventType: e.message === 'cancel' ? 'cancel' : 'error',
            message: e.message === 'cancel'
                ? 'Cancelled by user'
                : `Connection error: %yellow%${e.message}`,
            delay: 0,
            disconnectAfter: {
                stream: args.stream,
                server: args.server,
                socket: args.client,
                pick: args.pick,
                bar: args.bar,
            },
            padding: 4,
            fileInfo: args.fileInfo,
            executeLater: () => {
                if (e.message === 'cancel') {
                    args.candidate.failures.push(args.candidate.now);
                    args.candidate.queue = [];
                    if (fs.existsSync(args.fileInfo.filePath)) {
                        fs.unlinkSync(args.fileInfo.filePath);
                    }
                    this.emit('next', args.candidate, this.verbose);
                }
                else {
                    this.redownload(args.candidate, args.fileInfo);
                }
            },
        });
    }
    onClose(args, e) {
        if (e && args.received !== args.fileInfo.length) {
            this.emit('debug', `xdccJS:: TCP_CLOSE_ERROR @ ${e}`);
            return;
        }
        this.print('%success% done.', 6);
        this.emit('debug', 'xdccJS:: TCP_CLOSE_SUCCESS');
        args.candidate.timeout.clear();
        args.candidate.success.push(args.fileInfo.file);
        if (args.server && args.pick) {
            args.server.close(() => {
                this.portInUse = this.portInUse.filter((p) => p !== args.pick);
            });
        }
        args.stream.end();
        this.emit('downloaded', args.fileInfo);
        args.candidate.emit('downloaded', args.fileInfo);
        this.emit('next', args.candidate, this.verbose);
    }
    onData(args, data, sendBuffer) {
        if (args.received === 0) {
            args.candidate.timeout.clear();
            if (!this.path) {
                args.candidate.emit('pipe', args.stream, args.fileInfo);
                this.emit('pipe', args.stream, args.fileInfo);
            }
        }
        args.stream.write(data);
        args.received += data.length;
        if (args.bufferType === '64bit') {
            sendBuffer.writeBigUInt64BE(BigInt(args.received), 0);
        }
        if (args.bufferType === '32bit') {
            sendBuffer.writeUInt32BE(args.received, 0);
        }
        if (this.verbose && args.bar)
            args.bar.tick(data.length);
        if (!args.client?.destroyed && args.client?.writable) {
            args.client.write(sendBuffer);
        }
        args.candidate.emit('downloading', args.fileInfo, args.received, (args.received / args.fileInfo.length) * 100);
        this.emit('downloading', args.fileInfo, args.received, (args.received / args.fileInfo.length) * 100);
    }
    static setupProgressBar(len) {
        return new ProgressBar(''.padStart(6) + ctcp_parser_1.CtcpParser.replace(':roll [:bar] ETA: :eta @ :rate - :percent'), {
            complete: '=',
            incomplete: ' ',
            width: 20,
            total: len,
            clear: true,
            renderThrottle: 100,
        });
    }
}
exports.default = Downloader;
