"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable no-unused-vars */
/* eslint-disable max-classes-per-file */
const eventemitter3_1 = require("eventemitter3");
const bridge_1 = require("./bridge");
class XDCC extends eventemitter3_1.EventEmitter {
    irc;
    /**
     * Change config
     */
    config;
    action;
    ban;
    banlist;
    changeNick;
    channel;
    ctcpRequest;
    ctcpResponse;
    invite;
    join;
    mode;
    notice;
    part;
    ping;
    raw;
    rawString;
    say;
    sendMessage;
    setTopic;
    unban;
    who;
    whois;
    whowas;
    constructor(params) {
        // eslint-disable-next-line constructor-super
        super();
        this.irc = new bridge_1.default(params);
        this.listenCustomEvents();
        this.listenNativeEvents();
        this.config = this.irc.config.bind(this.irc);
        this.action = this.irc.action.bind(this.irc);
        this.ban = this.irc.ban.bind(this.irc);
        this.banlist = this.irc.banlist.bind(this.irc);
        this.changeNick = this.irc.changeNick.bind(this.irc);
        this.channel = this.irc.channel.bind(this.irc);
        this.ctcpRequest = this.irc.ctcpRequest.bind(this.irc);
        this.ctcpResponse = this.irc.ctcpResponse.bind(this.irc);
        this.invite = this.irc.invite.bind(this.irc);
        this.join = this.irc.join.bind(this.irc);
        this.mode = this.irc.mode.bind(this.irc);
        this.notice = this.irc.notice.bind(this.irc);
        this.part = this.irc.part.bind(this.irc);
        this.ping = this.irc.ping.bind(this.irc);
        this.quit = this.irc.quit.bind(this.irc);
        this.raw = this.irc.raw.bind(this.irc);
        this.rawString = this.irc.rawString.bind(this.irc);
        this.say = this.irc.say.bind(this.irc);
        this.sendMessage = this.irc.sendMessage.bind(this.irc);
        this.setTopic = this.irc.setTopic.bind(this.irc);
        this.unban = this.irc.unban.bind(this.irc);
        this.who = this.irc.who.bind(this.irc);
        this.whois = this.irc.whois.bind(this.irc);
        this.whowas = this.irc.whowas.bind(this.irc);
    }
    get user() {
        return this.irc.user;
    }
    get connected() {
        return this.irc.connected;
    }
    get candidates() {
        return Object.freeze(this.irc.candidates);
    }
    listenNativeEvents() {
        this.irc.on('irc error', (event) => this.emit('irc error', event));
        this.irc.on('join', (event) => this.emit('join', event));
        this.irc.on('kick', (event) => this.emit('kick', event));
        this.irc.on('message', (event) => this.emit('message', event));
        this.irc.on('mode', (event) => this.emit('mode', event));
        this.irc.on('nick in use', (event) => this.emit('nick in use', event));
        this.irc.on('nick invalid', (event) => this.emit('nick invalid', event));
        this.irc.on('notice', (event) => this.emit('notice', event));
        this.irc.on('part', (event) => this.emit('part', event));
        this.irc.on('quit', (event) => this.emit('quit', event));
        this.irc.on('raw', (event) => this.emit('raw', event));
        this.irc.on('raw socket connected', (event) => this.emit('raw socket connected', event));
        this.irc.on('registered', (event) => this.emit('registered', event));
        this.irc.on('server options', (event) => this.emit('server options', event));
        this.irc.on('socket close', (event) => this.emit('socket close', event));
        this.irc.on('socket connected', (event) => this.emit('socket connected', event));
    }
    listenCustomEvents() {
        this.irc.on('debug', (msg) => this.emit('debug', msg));
        this.irc.on('ready', () => {
            this.emit('ready');
        });
        this.irc.on('can-quit', () => {
            this.emit('can-quit');
        });
        this.irc.on('downloaded', (f) => {
            this.emit('downloaded', f);
        });
        this.irc.on('done', (job) => {
            const safeJob = job;
            this.emit('done', safeJob);
        });
        this.irc.on('pipe', (stream, f) => {
            this.emit('pipe', stream, f);
        });
        this.irc.on('error', (err) => {
            this.emit('error', err);
        });
        this.irc.on('downloading', (fileInfo, received, percentage) => {
            this.emit('downloading', fileInfo, received, percentage);
        });
    }
    /**
     * start jobs and download files
     * @param target bot name
     * @param packets pack number(s)
     * @param ipv6 use ipv6: only needed for passive bots
     * @example
     * ```js
     * xdccJS.on('ready', () => {
     *  xdccJS.download('XDCC|BLUE', '1-3, 8, 55')
     *  xdccJS.download('XDCC|RED', [1, 3, 10, 20])
     *  xdccJS.download('XDCC|YELLOW', 4)
     * })
     */
    download(target, packets, opts) {
        return this.irc.download(target, packets, opts);
    }
    /**
     * Quit IRC
     */
    quit() {
        this.irc.quit();
    }
    /**
     * Search jobs
     * @param bot Bot Name
     */
    async jobs(bot) {
        if (!bot)
            return this.candidates;
        return Object.freeze(this.candidates.find((b) => b.nick.localeCompare(bot, 'en', { sensitivity: 'base' }) === 0));
    }
}
exports.default = XDCC;
